# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import aiohttp.web

from api.handler.base import BaseHandler
from rac.cluster_manager import ClusterManager


class ClusterHandler(BaseHandler):
    @property
    def routes(self):
        return [
            aiohttp.web.post(self.build_route("/clusters"), self.create),
            aiohttp.web.get(self.build_route("/clusters"), self.read),
            aiohttp.web.get(self.build_route("/clusters/{cluster_id}"), self.read),
            aiohttp.web.put(self.build_route("/clusters"), self.update),
            aiohttp.web.delete(self.build_route("/clusters"), self.delete),
        ]

    async def create(self, request):
        pass

    async def read(self, request):
        cluster_id = request.match_info.get('cluster_id', None)
        # TODO: если задан cluster_id нужно возвращать один элемент, а не массив
        # TODO: возвращать 404 если элемент по cluster_id не найден
        return aiohttp.web.json_response(list(get_cluster(cluster_id)))

    async def update(self, request):
        pass

    async def delete(self, request):
        pass


def get_cluster(cluster_id=None):
    for cluster in ClusterManager().list():
        if cluster_id:
            if cluster.id == cluster_id:
                yield cluster.to_dict()
        else:
            yield cluster.to_dict()



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class Session:
    id = ""
    user = ""
    app = ""
    hibernate = False

    def __init__(self, session_id):
        self.id = session_id.strip()

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    @staticmethod
    def from_dict(params):
        session = Session(params["session"])
        session.user = params["user-name"]
        session.app = params["app-id"]
        session.hibernate = params["hibernate"] == "yes"
        return session

    def to_dict(self):
        return {
            "id": self.id,
            "user": self.user,
            "app": self.app,
            "hibernate": self.hibernate,
        }

# statistics: {
#     duration: {},
#     blocked: {
#         by_dbms: 0,
#         by_ls: 0,
#     },
#     bytes: {
#         all: 0,
#         last_5_min: 0
#     },
#     calls: {
#         last5min: 0,
#         all: 0,
#     }
#     memory: {
#         current: 0,
#         last5min: 0,
#         total: 0,
#     }
#     read: {
#         current: 0,
#         last5min: 0,
#         total: 0,
#     },
#     write: {
#         current: 0,
#         last5min: 0,
#         total: 0,
#     }
# }



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os

import hydra_agent.utils.system
from hydra_agent.config import Config
from hydra_agent.ring import Ring


class LicenseManager(Ring):
    def __init__(self, config: Config):
        super().__init__(config)

    def activate(self, license_info, serial, pin, previous_pin="", path=""):
        """Activates license"""

        args = [self.path, "license", "activate"]
        for k, v in license_info.items():
            if v.strip():
                args.append(f"--{k.replace('_','-')}")
                args.append(v.strip())
        args.extend(["--serial", serial.strip()])
        args.extend(["--pin", pin.strip()])
        if previous_pin:
            args.extend(["--previous-pin", previous_pin.strip()])
        if path:
            args.extend(["--path", path.strip()])
        self._run_command(args)
        return True

    def get(self, name, path=""):
        """Returns license file"""

        temp_file = hydra_agent.utils.system.temp_file_name()
        args = [self.path, "license", "get", "--name", name.strip(), "--license", temp_file]
        if path:
            args.extend(["--path", path.strip()])
        try:
            self._run_command(args)
            with open(temp_file, "rb") as f:
                lic = f.read()
        finally:
            if os.path.exists(temp_file):
                os.remove(temp_file)
        return lic

    def info(self, name, path=""):
        """Returns license info"""

        args = [self.path, "license", "info", "--name", name.strip()]
        if path:
            args.extend(["--path", path.strip()])
        return self._run_command(args)

    def list(self, path=""):
        """Returns list of licenses"""

        args = [self.path, "license", "list"]
        if path:
            args.extend(["--path", path.strip()])
        r = self._run_command(args)
        return r.split()

    def put(self, license, path=""):
        """Adds license file to storage"""

        license_file = hydra_agent.utils.system.temp_file_name()
        with open(license_file, "wb") as f:
            f.write(license)
        args = [self.path, "license", "put", "--license", license_file]
        if path:
            args.extend(["--path", path.strip()])
        try:
            self._run_command(args)
        finally:
            if os.path.exists(license_file):
                os.remove(license_file)
        return True

    def remove(self, name, path=""):
        """Removes license file from storage"""

        args = [self.path, "license", "remove", "--name", name.strip(), "--all"]
        if path:
            args.extend(["--path", path.strip()])
        self._run_command(args)
        return True

    def validate(self, name, path=""):
        """Validates license"""

        args = [self.path, "license", "validate", "--name", name.strip()]
        if path:
            args.extend(["--path", path.strip()])
        self._run_command(args)
        return True



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os.path
import re

from hydra_agent.utils.system import run_command, is_windows


class Ring:
    def __init__(self, config):
        self.path = os.path.join(config.path, "ring" + (".cmd" if is_windows() else ""))
        self.java = config.java

    @property
    def version(self):
        return run_command([self.path, "--version"])

    @property
    def modules(self):
        result = run_command([self.path, "help", "modules"])
        p = re.compile(r"[@:-]")

        modules = []
        for i in result.split("\n")[1:]:
            if not i.strip():
                continue
            modules.append(tuple(map(str.strip, p.split(i))))
        return modules

    def _run_command(self, args):
        # TODO: запуск под sudo
        os.environ["JAVA_HOME"] = self.java
        return run_command(args)



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import platform
import subprocess
import sys
from tempfile import NamedTemporaryFile


def is_linux():
    return platform.system() == "Linux"


def is_windows():
    return platform.system() == "Windows"


def is_mac():
    return platform.system() == "Darwin"


def temp_file_name():
    with NamedTemporaryFile(delete=False) as tmp:
        return tmp.name


def run_command(args):
    r = subprocess.run(
        args,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        universal_newlines=True,
        check=True,
        encoding=encoding()
    )
    return r.stdout.strip()


def encoding():
    if is_windows():
        import ctypes
        return f"cp{ctypes.windll.kernel32.GetOEMCP()}"
    else:
        return (sys.stdout.encoding if sys.stdout.isatty() else
                sys.stderr.encoding if sys.stderr.isatty() else
                sys.getfilesystemencoding() or "utf-8")



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import yaml

from hydra_agent.config.api_config import ApiConfig
from hydra_agent.config.rac_config import RacConfig
from hydra_agent.config.ring_config import RingConfig
from hydra_agent.config.v8_config import V8Config

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader


class Config:
    def __init__(self, path="", source=""):
        self._load(path, source)

    def __str__(self):
        return f"rac: {self.rac}, ring: {self.ring}, api: {self.api}, v8: {self.v8}"

    def reload(self, path="", source=""):
        self._load(path, source)

    def _load(self, path, source):
        config = self._config(path, source)

        self.rac = self._rac_config(config)
        self.ring = self._ring_config(config)
        self.api = self._api_config(config)
        self.v8 = self._v8_config(config)

    @staticmethod
    def _rac_config(config):
        path = ""
        host = "localhost"
        port = 1545
        java = ""
        if "rac" in config:
            if "path" in config["rac"]:
                path = config["rac"]["path"]
            if "host" in config["rac"]:
                host = config["rac"]["host"]
            if "port" in config["rac"]:
                port = config["rac"]["port"]

        return RacConfig(path, host, port)

    @staticmethod
    def _ring_config(config):
        path = ""
        java = ""
        if "ring" in config:
            if "path" in config["ring"]:
                path = config["ring"]["path"]
            if "java" in config["ring"]:
                java = config["ring"]["java"]
        return RingConfig(path, java)

    @staticmethod
    def _api_config(config):
        host = "localhost"
        port = 9523
        debug = False
        if "api" in config:
            if "host" in config["api"]:
                host = config["api"]["host"]
            if "port" in config["api"]:
                port = config["api"]["port"]
            if "debug" in config["api"]:
                debug = config["api"]["debug"]
        return ApiConfig(host, port, debug)

    @staticmethod
    def _v8_config(config):
        path = ""
        if "v8" in config:
            if "path" in config["v8"]:
                path = config["v8"]["path"]
        return V8Config(path)

    @staticmethod
    def _config(path, source):
        if source:
            config = yaml.load(source, Loader)
        elif path:
            with open(path) as f:
                config = yaml.load(f, Loader)
        else:
            config = {}

        return config



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import platform

import aiohttp.web

from hydra_agent.__version__ import hydra_agent


class HeartbeatHandler(aiohttp.web.View):
    async def get(self):
        return aiohttp.web.json_response(heartbeat())


def heartbeat():
    data = {
        "version": {
            "hydra_agent": hydra_agent,
        },
        "os": os_info(),
        "cpu": cpu_count(),
    }

    return data


def os_info():
    r = platform.uname()
    result = {
        "system": r.system,
        "node": r.node,
        "release": r.release,
        "version": r.version,
        "architecture": platform.architecture()[0]
    }
    return result


def cpu_count():
    return os.cpu_count()



# This file is part of HYDRA - cross-platform remote administration
# system for 1C:Enterprise (https://github.com/vbondarevsky/hydra_agent).
# Copyright (C) 2017  Vladimir Bondarevskiy.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import optparse

from hydra_agent import config
from hydra_agent.api.main import run_server


def run():
    parser = optparse.OptionParser()
    parser.add_option("-c", "--config", dest="config", help="path to config file")

    (options, args) = parser.parse_args()

    if options.config:
        config.reload(path=options.config)

    run_server()


if __name__ == "__main__":
    run()
